
let pi = 4. *. atan 1.

(* ボックス＝ミュラー法での正規乱数の生成 *)
let nrandom () =
  let r1 = Random.float 1.0 in
  let r2 = Random.float 1.0 in
  let u1 = sqrt (-2. *. log r1) *. sin (2. *. pi *. r2) in
  let u2 = sqrt (-2. *. log r1) *. cos (2. *. pi *. r2) in
  (u1, u2)

let last_elem x_arr =
  let n = Array.length x_arr - 1 in
  x_arr.(n)

(* 確率微分方程式の解 *)
let rec sde_solution n_max x_arr f g h n =
  let (z1, z2) = nrandom () in
  let x0 = last_elem x_arr in
  let x1 = x0 +. (f x0) *. h +. (g x0) *. z1 *. sqrt h in
  let x2 = x1 +. (f x1) *. h +. (g x1) *. z2 *. sqrt h in
  let x_arr_next = Array.append x_arr [| x1; x2 |] in
  if n + 1 >= n_max then x_arr_next
  else sde_solution n_max x_arr_next f g h (n+1)


let () =
  Random.self_init ();
  let x0 = [|0.|] in
  let f x = -.x in
  let g x = 1. in
  let x_arr = sde_solution 50 x0 f g 0.01 0 in
  Array.iter (fun x -> print_endline (string_of_float x)) x_arr

